@@ -67,7 +67,21 @@ def test_prefix(c, s, a, b):
              scheduler_kwargs={'services': {('bokeh', 0):  BokehScheduler}})
 def test_prometheus(c, s, a, b):
     pytest.importorskip('prometheus_client')
+    from prometheus_client.parser import text_string_to_metric_families
+
     http_client = AsyncHTTPClient()
-    response = yield http_client.fetch('http://localhost:%d/metrics'
-                                       % s.services['bokeh'].port)
-    assert response.code == 200
+
+    # request data twice since there once was a case where metrics got registered multiple times resulting in
+    # prometheus_client errors
+    for _ in range(2):
+        response = yield http_client.fetch('http://localhost:%d/metrics'
+                                           % s.services['bokeh'].port)
+        assert response.code == 200
+        assert response.headers['Content-Type'] == 'text/plain; version=0.0.4'
+
+        txt = response.body.decode('utf8')
+        families = {
+            familiy.name
+            for familiy in text_string_to_metric_families(txt)
+        }
+        assert 'dask_scheduler_workers' in families
