@@ -30,13 +30,11 @@ def test_defaults(loop):
         def f():
             # Default behaviour is to listen on all addresses
             yield [
-                assert_can_connect_from_everywhere_4_6(8786, 2.0),  # main port
+                assert_can_connect_from_everywhere_4_6(8786, 5.0),  # main port
             ]
 
-        loop.run_sync(f)
-
         with Client('127.0.0.1:%d' % Scheduler.default_port, loop=loop) as c:
-            pass
+            c.sync(f)
 
     with pytest.raises(Exception):
         requests.get('http://127.0.0.1:8787/status/')
@@ -50,13 +48,12 @@ def test_hostport(loop):
         def f():
             yield [
                 # The scheduler's main port can't be contacted from the outside
-                assert_can_connect_locally_4(8978, 2.0),
+                assert_can_connect_locally_4(8978, 5.0),
             ]
 
-        loop.run_sync(f)
-
         with Client('127.0.0.1:8978', loop=loop) as c:
             assert len(c.ncores()) == 0
+            c.sync(f)
 
 
 def test_no_bokeh(loop):
